import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def edges_from_elem(ele): ...
def elems_depth_measure(ele_dst, ele_src, other_edges_over_cb):
    """Returns·ele_dst vert depths from ele_src, aligned with ele_dst verts."""

    ...

def elems_depth_search(ele_init, depths, other_edges_over_cb, results_init=None):
    """List of depths -> List of elems that match those depths."""

    ...

def find_next(ele_dst, ele_src): ...
def other_edges_over_edge(e): ...
def other_edges_over_face(e): ...
def select_next(bm, report): ...
def select_prev(bm, report): ...
def verts_from_elem(ele): ...
