import typing
import collections.abc
import typing_extensions
from . import progress_report as progress_report

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")
