import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def check(check_ctxt, msgs, key, msgsrc, settings):
    """Performs a set of checks over the given key (context, message)..."""

    ...

def dump_addon_bl_info(msgs, reports, module, settings): ...
def dump_addon_messages(addon_module_name, do_checks, settings): ...
def dump_asset_messages(msgs, reports, settings): ...
def dump_extension_metadata(msgs, reports, settings): ...
def dump_messages(do_messages, do_checks, settings): ...
def dump_preset_messages(msgs, reports, settings): ...
def dump_py_messages(msgs, reports, addons, settings, addons_only=False): ...
def dump_py_messages_from_files(msgs, reports, files, settings): ...
def dump_rna_messages(msgs, reports, settings, verbose=False):
    """Dump into messages dict all RNA-defined UI messages (labels en tooltips)."""

    ...

def dump_src_messages(msgs, reports, settings): ...
def dump_template_messages(msgs, reports, settings): ...
def init_spell_check(settings, lang="en_US"): ...
def main(): ...
def print_info(reports, pot): ...
def process_msg(msgs, msgctxt, msgid, msgsrc, reports, check_ctxt, settings): ...
