import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def do_clear_previews(do_objects, do_collections, do_scenes, do_data_intern): ...
def do_previews(do_objects, do_collections, do_scenes, do_data_intern): ...
def ids_nolib(bids): ...
def ids_nolib_with_preview(bids): ...
def main(): ...
def rna_backup_gen(data, include_props=None, exclude_props=None, root=()): ...
def rna_backup_restore(data, backup): ...
