import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def indent(levels): ...
def keyconfig_export_as_data(wm, kc, filepath, *, all_keymaps=False): ...
def keyconfig_import_from_data(
    name, keyconfig_data, *, keyconfig_version=(0, 0, 0)
): ...
def keyconfig_init_from_data(kc, keyconfig_data): ...
def keyconfig_merge(kc1, kc2):
    """note: kc1 takes priority over kc2"""

    ...

def keymap_init_from_data(km, km_items, is_modal=False): ...
def kmi_args_as_data(kmi): ...
def repr_f32(f): ...
def round_float_32(f): ...
